-- Author: VertexFloat
-- Date: 08.08.2022
-- Version: Farming Simulator 22, 1.0.0.0
-- Copyright (C): VertexFloat, All Rights Reserved
-- AttachFix main class

AttachFix = {}

local AttachFix_mt = Class(AttachFix)

function AttachFix.new(manualAttach)
    local self = setmetatable({}, AttachFix_mt)

    if manualAttach ~= nil then
        self:setImplementsLoweredOnAttach(manualAttach.isEnabled)
    else
        self:setImplementsLoweredOnAttach(false)
    end

    g_messageCenter:subscribe(BuyVehicleEvent, self.onNewVehicleInit, self)
    g_messageCenter:subscribe(MessageType.VEHICLE_RESET, self.onNewVehicleInit, self)

    return self
end

function AttachFix:update(dt)
    local isNotLowered = false

    for _, vehicle in pairs(g_currentMission.vehicles) do
        if vehicle ~= nil and g_currentMission.controlledVehicle ~= nil then
            if vehicle == g_currentMission.controlledVehicle then
                if SpecializationUtil.hasSpecialization(AttacherJoints, vehicle.specializations) then
                    local info = vehicle.spec_attacherJoints.attachableInfo
                    local dynamicMountedObject = self:getMountedObject()

                    if info.attachable ~= nil then
                        if info.attachable == dynamicMountedObject then
                            isNotLowered = true
                        end
                    end
                end
            end
        end
    end

    AttachFix.isNotLowered = isNotLowered
end

function AttachFix:setImplementsLoweredOnAttach(isManualAttachEnabled)
    if isManualAttachEnabled == nil then
        isManualAttachEnabled = false
    end

    for _, vehicle in pairs(g_currentMission.vehicles) do
        if vehicle ~= nil then
            if SpecializationUtil.hasSpecialization(Attachable, vehicle.specializations) then
                for key, value in pairs(vehicle.spec_attachable) do
                    if key == "inputAttacherJoints" then
                        for _, inputAttacherJoint in pairs(value) do
                            if isManualAttachEnabled then
                                inputAttacherJoint.isDefaultLowered = false

                                for jointType, _ in pairs(AttachFix.allowedJointTypes) do
                                    if jointType == inputAttacherJoint.jointType then
                                        inputAttacherJoint.isDefaultLowered = true
                                    end
                                end
                            else
                                inputAttacherJoint.isDefaultLowered = true
                            end
                        end
                    end
                end
            end
        end
    end
end

function AttachFix:getMountedObject()
    for _, vehicle in pairs(g_currentMission.vehicles) do
        if vehicle ~= nil then
            if SpecializationUtil.hasSpecialization(DynamicMountAttacher, vehicle.specializations) then
                for _, dynamicMountedObject in pairs(vehicle.spec_dynamicMountAttacher.dynamicMountedObjects) do
                    return dynamicMountedObject
                end
            end
        end
    end
end

function AttachFix:onNewVehicleInit()
    if g_modIsLoaded["FS22_manualAttach"] then
        self:setImplementsLoweredOnAttach(_G["FS22_manualAttach"].g_manualAttach.isEnabled)
    else
        self:setImplementsLoweredOnAttach(false)
    end
end

function AttachFix:onManualAttachModeChanged()
    AttachFix:setImplementsLoweredOnAttach(_G["FS22_manualAttach"].g_manualAttach.isEnabled)
end

function AttachFix:deleteMap()
    g_messageCenter:unsubscribe(BuyVehicleEvent)
    g_messageCenter:unsubscribe(MessageType.VEHICLE_RESET)

    removeModEventListener(AttachFix)
end

addModEventListener(AttachFix)

local function getAttacherVehicleJointDescIndex(info)
    for a, b in pairs(info) do
        if a == "spec_attacherJoints" then
            for c, d in pairs(b.attachableInfo) do
                if c == "attacherVehicleJointDescIndex" then
                    return d
                end
            end
        end
    end
end

local function onStartMission()
    if g_modIsLoaded["FS22_manualAttach"] then
        _G["FS22_manualAttach"].g_manualAttach.onManualAttachModeChanged = Utils.appendedFunction(_G["FS22_manualAttach"].g_manualAttach.onManualAttachModeChanged, AttachFix.onManualAttachModeChanged)

        AttachFix.allowedJointTypes = _G["FS22_manualAttach"].g_manualAttach.AUTO_ATTACH_JOINTYPES

        AttachFix.new(_G["FS22_manualAttach"].g_manualAttach)
    else
        AttachFix.new(nil)
    end
end

Mission00.onStartMission = Utils.prependedFunction(Mission00.onStartMission, onStartMission)

local function attachImplementFromInfo(info)
    for _, vehicle in pairs(g_currentMission.vehicles) do
        if vehicle ~= nil and g_currentMission.controlledVehicle ~= nil then
            if vehicle == g_currentMission.controlledVehicle then
                if SpecializationUtil.hasSpecialization(Combine, vehicle.specializations) and SpecializationUtil.hasSpecialization(AttacherJoints, vehicle.specializations) then
                    local jointDescIndex = getAttacherVehicleJointDescIndex(info)

                    if AttachFix.isNotLowered then
                        vehicle:setJointMoveDown(jointDescIndex, false, true)
                    end
                end
            end
        end
    end
end

AttacherJoints.attachImplementFromInfo = Utils.appendedFunction(AttacherJoints.attachImplementFromInfo, attachImplementFromInfo)